import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { BackendQueryFormat } from '../../querybuilder';

/** Message format of the error message from the backend */
export type ErrorMessage = {
  errorStatus: any;
  queryID: any;
};

/** Cache type */
export type SessionCacheI = {
  currentDatabase?: string;
  databases: string[];
};

// Define the initial state using that type
export const initialState: SessionCacheI = {
  currentDatabase: undefined,
  databases: [],
};

export const sessionSlice = createSlice({
  name: 'session',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    updateCurrentDatabase(state, action: PayloadAction<string>) {
      state.currentDatabase = action.payload;
    },
    updateDatabaseList(state, action: PayloadAction<string[]>) {
      console.debug('Updating database list', action);
      state.databases = action.payload;
      if (state.databases.length > 0) {
        if (!state.currentDatabase || !state.databases.includes(state.currentDatabase)) state.currentDatabase = state.databases[0];
        else state.currentDatabase = undefined;
      }
    },
  },
});

export const { updateCurrentDatabase, updateDatabaseList } = sessionSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const sessionCacheState = (state: RootState) => state.sessionCache;

export default sessionSlice.reducer;
